// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file AMDiS_fwd.h */


#ifndef AMDIS_AMDIS_FWD_INCLUDE
#define AMDIS_AMDIS_FWD_INCLUDE

#include <boost/numeric/mtl/mtl.hpp>
#include "OpenMP.h"

namespace AMDiS {

  class AdaptInfo;
  class AdaptStationary;
  class Assembler;
  class BasisFunction; 
  class BoundaryManager;
  class CGSolver;
  class CoarseningManager;
  class CouplingIterationInterface;
  class DiagonalPreconditioner; 
  class DOFAdmin; 
  class DOFContainer;
  class DOFIndexedBase;
  class DOFMatrix;
  class DOFVectorDOF;
  class Element;
  class ElementDofIterator;
  class ElInfo; 
  class ElMatrixInfo;
  class Estimator;
  class FastQuadrature;
  class FillInfo; 
  class FileWriterInterface;
  class FiniteElemSpace; 
  class Flag;
  class IdentityPreconditioner; 
  class InteriorBoundary;
  class LeafDataPeriodic;
  class LevelAdmin;
  class Line;
  class MacroElement;
  class MacroInfo;
  class Marker;
  class Mesh; 
  class MeshStructure;
  class OEMSolver;
  class Operator;
  class OperatorTerm;
  class Parametric;
  class PeriodicBC;
  class ProblemInstat;
  class ProblemInstatScal;
  class ProblemInstatVec;
  class ProblemIterationInterface;
  class ProblemStatBase;
  class ProblemStatSeq;
  class ProblemTimeInterface;
  class Projection;
  class PreconditionerScal;
  class Quadrature; 
  class Q00PsiPhi;
  class Q0Psi;
  class Q10PsiPhi;
  class Q01PsiPhi;
  class Q1Psi;
  class RCNeighbourList;
  class RefinementManager;
  class RosenbrockAdaptInstationary;
  class RosenbrockStationary;
  class RobinBC;
  class SubElInfo;
  class SurfaceOperator;
  class SMIAdapter;
  class SystemVector;
  class Tetrahedron;
  class TraverseStack;
  class Triangle;
  class VertexInfo;
  class VertexVector;

#ifdef HAVE_PARALLEL_DOMAIN_AMDIS
  class ElementObjectDatabase;
  class FeSpaceDofMap;
  class MatrixNnzStructure;
  class MeshLevelData;
  class PetscSolverFeti;
  class PetscSolverFetiDebug;
#endif

  struct BoundaryObject;
  struct AtomicBoundary;

  template<typename ReturnType, typename ArgumentType> class AbstractFunction;
  template<typename ReturnType, 
	   typename ArgumentType1, 
	   typename ArgumentType2>                     class BinaryAbstractFunction;
  template<typename ReturnType, 
	   typename ArgumentType1, 
	   typename ArgumentType2,
	   typename ArgumentType3>                     class TertiaryAbstractFunction;
  template<typename ReturnType, 
	   typename ArgumentType1, 
	   typename ArgumentType2, 
	   typename ArgumentType3,
	   typename ArgumentType4>                     class QuartAbstractFunction;

  template<typename T>                                 class DOFIndexed;
  template<typename T>                                 class DOFVectorBase;
  template<typename T>                                 class DOFVector;
  template<typename T>                                 class DimVec;
  template<typename T>                                 class DimMat;
  template<typename ITLSolver>                         class ITL_OEMSolver;
  template<typename T, typename Vec, typename Mat >    class ITL_Preconditioner;
  template<typename T>                                 class Matrix;
  template<typename T>                                 class MatrixOfFixVecs;
  template<typename T>                                 class MatVecMultiplier;
  template<typename T>                                 class SolverMatrix;
  template<typename T>                                 class Vector;
  template<typename T>                                 class WorldVector;
  template<typename T>                                 class WorldMatrix;
  template<typename T>                                 class VectorOfFixVecs;


  typedef mtl::dense2D<double>              ElementMatrix;
  typedef mtl::dense_vector<double>         ElementVector;

} // namespace AMDiS

#endif // AMDIS_AMDIS_FWD_INCLUDE
